use crate::{
    decrypted_account::DecryptedAccount,
    entity::prelude::Account,
    handlers::{utils::package_handler, MainDialogue, State},
};
use futures::TryStreamExt;
use sea_orm::DatabaseConnection;
use serde_json::{json, to_string_pretty};
use std::sync::Arc;
use teloxide::{adaptors::Throttle, prelude::*, types::InputFile};
use tokio::task::JoinSet;

async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    previous: Message,
    master_pass: String,
) -> crate::Result<()> {
    let _ = bot.delete_message(previous.chat.id, previous.id).await;
    let master_pass: Arc<str> = master_pass.into();
    let user_id = msg.from().unwrap().id.0;
    let mut join_set = JoinSet::new();
    let mut accounts = Vec::new();
    Account::get_all(user_id, &db)
        .await?
        .try_for_each(|account| {
            let master_pass = Arc::clone(&master_pass);
            join_set.spawn_blocking(move || DecryptedAccount::from_account(account, &master_pass));
            async { crate::Result::Ok(()) }
        })
        .await?;
    drop(master_pass);
    while let Some(account) = join_set.join_next().await.transpose()?.transpose()? {
        accounts.push(account)
    }
    accounts.sort_by(|this, other| this.name.cmp(&other.name));
    let json = to_string_pretty(&json!({ "accounts": accounts }))?;
    let file = InputFile::memory(json).file_name("accounts.json");
    bot.send_document(msg.chat.id, file).await?;
    dialogue.exit().await?;
    Ok(())
}

pub async fn export(bot: Throttle<Bot>, msg: Message, dialogue: MainDialogue) -> crate::Result<()> {
    let previous = bot
        .send_message(msg.chat.id, "Send a master password to export the accounts")
        .await?;
    dialogue
        .update(State::GetMasterPass(package_handler(
            move |bot, msg, db, dialogue, master_pass| {
                get_master_pass(bot, msg, db, dialogue, previous, master_pass)
            },
        )))
        .await?;
    Ok(())
}
